package org.vamsas.test.simpleclient;

import java.io.File;
import java.io.ObjectOutputStream;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.vamsas.client.simpleclient.SimpleDocument;
import org.vamsas.client.simpleclient.VamsasArchiveReader;
import org.vamsas.objects.core.ApplicationData;
import org.vamsas.objects.core.User;
import org.vamsas.objects.core.VAMSAS;
import org.vamsas.objects.core.VamsasDocument;
import org.vamsas.test.objects.Core;

public class VamsasArchive {
  /**
   * test the org.vamsas.simpleclient.vamsasArchive class
   */
  static Log log = LogFactory.getLog(VamsasArchive.class);
  public static ApplicationData makeDemoAppdata(org.vamsas.client.simpleclient.VamsasArchive va, String apname, String userName, String userOrg) {
    if (va==null)
      return null;
    VamsasArchiveReader vread=null;
    try {
      vread = va.getOriginalArchiveReader();
    }
    catch (Exception e) {
      log.error("Failed to get original archive reader!",e);
      return null;
    }
    ApplicationData appdata = new ApplicationData();
    appdata.setName("org.vamsas.test.simpleclient.VamsasArchive");
    appdata.setData(new String("this is some test data.").getBytes());
    User apuser = new User();
    apuser.setFullname(userName);
    apuser.setOrganization(userOrg);
    String appdata_ref = "vamsas:"+apname+"/"+apuser.getOrganization()+"/"+apuser.getFullname();
    SimpleDocument sdoc = new SimpleDocument("test.simpleclient.VamsasArchive");
    if (vread!=null) {
      VamsasDocument orignalnew;
      try {
        orignalnew = sdoc.getVamsasDocument(vread);
        log.info("*** Dump follows ***");
        
        ArchiveReports.reportDocument(orignalnew, vread, false);
        log.info("*** Dump precedes ***");
      } catch (Exception e) {
        log.info("makeDemoAppdata: Problems accessing original document");
      }
      
      log.info("Reading (and avoiding references to) original data");
      if (vread.getAppdataStream(appdata_ref)!=null) {
        // transfer over
        try {
          va.transferAppDataEntry(appdata_ref);
        } catch (Exception e) {
          log.warn("Exception when transferring appdata reference : "+appdata_ref, e);
        }
        int i=0;
        while (vread.getAppdataStream(appdata_ref+"/"+Integer.toString(++i))!=null) {
          try {
            // copy over another duplicate.
            va.transferAppDataEntry(appdata_ref+"/"+Integer.toString(i));
          } catch (Exception e) {
            log.warn("Exception when transferring appdata reference : "+appdata_ref, e);
          }
        }
        // this one must be unique!
        appdata_ref+="/"+Integer.toString(i);
      }
    }
    
    log.info("Adding new data stuff.");
    log.info("Writing an apdata reference using AppDataStream interface.");
    apuser.setDataReference(appdata_ref);
    appdata.addUser(apuser);
    appdata.setVersion("noggin");
    appdata.setUrn("program:/the.nog/");
    try {
      ObjectOutputStream ost = new ObjectOutputStream(va.getAppDataStream(appdata_ref));
      ost.writeObject(appdata);
      ost.close();
    } catch (Exception e) {
      log.warn("Couldn't write appdata reference "+appdata_ref);
    }
    return appdata;
  }
  public static void main(String args[]) {
    
    try {
      File av;
      if (args.length>0)
        av = new File(args[0]);
      else
        av = new File("test/vamsas.zip");
      log.info("Opening archive "+av);
      org.vamsas.client.simpleclient.VamsasArchive varchive = new org.vamsas.client.simpleclient.VamsasArchive(av, true);
      
      VAMSAS[] roots = (VAMSAS[]) varchive.getOriginalRoots();
      
      if (roots!=null) {
        log.info("Report on Original roots in archive:");
        ArchiveReports.rootReport(roots, true);
      }
      log.info("Getting current vamsas document.");
      VamsasDocument doc = varchive.getVamsasDocument();
      ArchiveReports.reportDocument(doc, varchive.getOriginalArchiveReader(), true); // not modified document so references will still be valid
      // do some stuff
      log.info("Retrieving backup");
      File backup = varchive.backupFile();
      if (backup==null)
        log.info(av+" is a New Archive.");
      else 
        log.info(av+" has been backed up as "+backup);
      File newf=new File(av.getAbsolutePath()+"_new.zip");
      if (newf.exists()) {
        int q=1;
        do {
          newf=new File(av.getAbsolutePath()+"_"+q+++"_new.zip");
        }
        while (newf.exists()); 
      } 
      log.info("Now writing new Archive "+newf.getAbsolutePath());
      org.vamsas.client.simpleclient.VamsasArchive va = new org.vamsas.client.simpleclient.VamsasArchive(newf, true);
      // open another and...
      ApplicationData appdata = makeDemoAppdata(va, 
          "org.vamsas.test.simpleclient.VamsasArchive", "arnold Bugger esq", "disOrganised");
      log.info("Preparing to write new document.");
      doc.addApplicationData(appdata);
      doc.addVAMSAS(Core.getDemoVamsas());
      // TODO: ensure all jar entries are transferred over (at least those that are referenced - make a flag for 'tidy zip manifest'.
      va.putVamsasDocument(doc); // gets stream and puts it.
      // TODO: verify that the vxml stream close method overridden ?
      va.closeArchive();
      
      log.info("Dump of new vamsas document :");
      log.info("Testing update: ");
      VamsasArchiveReader vreader = new VamsasArchiveReader(newf);
      SimpleDocument sdoc = new SimpleDocument("testing new vamsas write");
      ArchiveReports.reportDocument(sdoc.getVamsasDocument(vreader), vreader, true);
      
      // backup.delete(); // tidy up
      
      log.info("Now Cancelling write to original archive "+av);
      if (varchive.cancelArchive())
        log.info("Successfully cancelled.");
      else
        log.info("Didn't cancel.");
      
      log.info("Now testing archive update.");
      va = new org.vamsas.client.simpleclient.VamsasArchive(newf, true); // TODO - refactor Reader class to have deserializing helpers
      doc = va.getVamsasDocument();
      doc.addVAMSAS(Core.getDemoVamsas());
      doc.addApplicationData(makeDemoAppdata(va, 
          "org.vamsas.test.simpleclient.VamsasArchive", "another old Bugger esq", "rescinded"));
      if (va.transferRemainingAppDatas())
        log.info("Remain appdatas were transferred.");
      else
        log.warn("No appdatas were transferred. This is wrong.");
      va.putVamsasDocument(doc);
      va.closeArchive();
      log.info("Testing update: ");
      vreader = new VamsasArchiveReader(newf);
      sdoc = new SimpleDocument("testing vamsas update");
      ArchiveReports.reportDocument(sdoc.getVamsasDocument(vreader), vreader, true);
    } catch (Exception e) {
      e.printStackTrace(System.err);
    }
  }
}
